from __future__ import (absolute_import, division, print_function)

import unittest
from pyoos.collectors.usgs.usgs_rest import UsgsRest
from datetime import datetime, timedelta


class USGSTest(unittest.TestCase):

    def setUp(self):
        self.c = UsgsRest()

    def test_by_bbox(self):
        self.c.filter(bbox=(-87, 46, -85, 48))
        collection = self.c.collect()
        collection.calculate_bounds()

        # Returns 4 stations: 04044724, 04045500, 04046000, 04056500
        assert len(collection.elements) == 5
        assert sorted([x.uid for x in collection.elements]) == ['04044724', '04044755', '04045500', '04046000', '04056500']

        station = collection.elements[0]
        assert station.name == "AU TRAIN RIVER AT FOREST LAKE, MI"
        # Measures 2 variables
        assert len(station.get_unique_members()) == 2
        assert station.location.x == -86.8501514
        assert station.location.y == 46.34077908
        assert station.location.z == 0

        # Apply time filter as well
        starting = datetime(2013, 12, 31, 0, 0, 0)
        ending = starting + timedelta(hours=6)
        self.c.filter(start=starting, end=ending)
        collection = self.c.collect()
        collection.calculate_bounds()

        # Returns 4 stations: 04044724, 04045500, 04046000, 04056500
        assert len(collection.elements) == 4
        assert sorted([x.uid for x in collection.elements]) == ["04044724", "04045500", "04046000", "04056500"]

    def test_by_state(self):
        # Clear filters
        self.c.clear()
        # Add custom state filter
        self.c.filter(state="ri")
        collection = self.c.collect()

        # Returns 43 stations.
        # FIXME: This is a very flaky test!  The station number changed from
        # 41, to 42 and now 43. (And back to 42!)
        assert len(collection.elements) == 42

        station = collection.elements[0]
        assert station.name == "TEN MILE R., PAWTUCKET AVE. AT E. PROVIDENCE, RI"
        # Measures 2 variables
        assert len(station.get_unique_members()) == 2
        assert station.location.x == -71.3511649
        assert station.location.y == 41.83093376
        assert station.location.z == 0

    def test_by_site_code(self):
        # Clear filters
        self.c.clear()
        # Add custom state filter
        self.c.filter(features=['04001000'])
        collection = self.c.collect()
        collection.calculate_bounds()

        station = collection.elements[0]
        assert station.uid == '04001000'
        assert station.name == 'WASHINGTON CREEK AT WINDIGO, MI'
        assert station.location.x == -89.1459196999999932
        assert station.location.y == 47.9212792000000007
        assert station.location.z == 0



from __future__ import (absolute_import, division, print_function)
from six import string_types

import unittest
import csv
import io
from datetime import datetime

from owslib.swe.sensor.sml import SensorML

from pyoos.collectors.ndbc.ndbc_sos import NdbcSos


class NdbcSosTest(unittest.TestCase):

    def setUp(self):
        self.c = NdbcSos()

    def test_ndbc_server_id(self):
        assert self.c.server.identification.title == "National Data Buoy Center SOS"
        assert self.c.server.identification.service == 'OGC:SOS'
        assert self.c.server.identification.version == '1.0.0'
        assert self.c.server.identification.abstract == 'National Data Buoy Center SOS'
        # assert self.c.server.identification.keywords == ['Weather', 'Ocean Currents', 'Air Temperature', 'Water Temperature', 'Conductivity', 'Salinity', 'Barometric Pressure', 'Water Level', 'Waves', 'Winds', 'NDBC']
        assert self.c.server.identification.fees == 'NONE'
        assert self.c.server.identification.accessconstraints == 'NONE'

    def test_ndbc_describe_sensor(self):
        self.c.features = ['41012']
        response = self.c.metadata(output_format='text/xml;subtype="sensorML/1.0.1"')
        assert isinstance(response, list)
        assert isinstance(response[0], SensorML)

    def test_raw_ndbc_get_observation(self):
        self.c.start_time   = datetime.strptime("2012-10-01", "%Y-%m-%d")
        self.c.end_time     = datetime.strptime("2012-10-02", "%Y-%m-%d")
        self.c.features     = ['41012']
        self.c.variables    = ['air_pressure_at_sea_level']

        response = self.c.raw(responseFormat="text/csv").decode()
        assert isinstance(response, string_types)
        """
        station_id,sensor_id,"latitude (degree)","longitude (degree)",date_time,"depth (m)","air_pressure_at_sea_level (hPa)"
        urn:ioos:station:wmo:41012,urn:ioos:sensor:wmo:41012::baro1,30.04,-80.55,2012-10-01T00:50:00Z,0.00,1009.8
        """
        data = list(csv.DictReader(io.StringIO(response)))
        assert data[0]['station_id'] == 'urn:ioos:station:wmo:41012'
        assert data[0]['sensor_id'] == 'urn:ioos:sensor:wmo:41012::baro1'
        assert data[0]['date_time'] == "2012-10-01T00:50:00Z"
        assert data[0]['depth (m)'] == "0.00"
        assert data[0]['air_pressure_at_sea_level (hPa)'] == "1009.8"

    def test_raw_ndbc_get_observation_all_stations(self):
        self.c.start_time   = datetime.strptime("2012-10-01", "%Y-%m-%d")
        self.c.end_time     = datetime.strptime("2012-10-02", "%Y-%m-%d")
        # TODO: This should not return all stations in the future.  We should make multiple requests.
        self.c.features     = ['32st0', '41012']  # Triggers network-all
        self.c.variables    = ['air_pressure_at_sea_level']

        response = self.c.raw(responseFormat="text/csv").decode()
        assert isinstance(response, string_types)

        data = list(csv.DictReader(io.StringIO(response)))
        stations = list(set([x['station_id'] for x in data]))
        # 265 stations measured air_pressure that day
        assert len(stations) == 265

        """
        station_id,sensor_id,"latitude (degree)","longitude (degree)",date_time,"depth (m)","air_pressure_at_sea_level (hPa)"
        urn:ioos:station:wmo:32st0,urn:ioos:sensor:wmo:32st0::baro1,-19.713,-85.585,2012-10-01T00:00:00Z,,1019.0
        """
        assert data[0]['station_id'] == 'urn:ioos:station:wmo:32st0'
        assert data[0]['sensor_id'] == 'urn:ioos:sensor:wmo:32st0::baro1'
        assert data[0]['date_time'] == "2012-10-01T00:00:00Z"
        assert data[0]['depth (m)'] == ""
        assert data[0]['air_pressure_at_sea_level (hPa)'] == "1019.0"

    def test_raw_ndbc_get_observation_no_stations(self):
        self.c.start_time   = datetime.strptime("2012-10-01", "%Y-%m-%d")
        self.c.end_time     = datetime.strptime("2012-10-02", "%Y-%m-%d")
        self.c.features     = []  # Triggers network-all
        self.c.variables    = ['air_pressure_at_sea_level']

        response = self.c.raw(responseFormat="text/csv").decode()
        assert isinstance(response, string_types)

        data = list(csv.DictReader(io.StringIO(response)))
        stations = list(set([x['station_id'] for x in data]))
        # 265 stations measured air_pressure that day
        assert len(stations) == 265

        """
        station_id,sensor_id,"latitude (degree)","longitude (degree)",date_time,"depth (m)","air_pressure_at_sea_level (hPa)"
        urn:ioos:station:wmo:32st0,urn:ioos:sensor:wmo:32st0::baro1,-19.713,-85.585,2012-10-01T00:00:00Z,,1019.0
        """
        assert data[0]['station_id'] == 'urn:ioos:station:wmo:32st0'
        assert data[0]['sensor_id'] == 'urn:ioos:sensor:wmo:32st0::baro1'
        assert data[0]['date_time'] == "2012-10-01T00:00:00Z"
        assert data[0]['depth (m)'] == ""
        assert data[0]['air_pressure_at_sea_level (hPa)'] == "1019.0"



from __future__ import (absolute_import, division, print_function)
from six import string_types

from pyoos.collectors.collector import Collector
from pyoos.parsers.ioos.get_observation import IoosGetObservation
from owslib.ows import ExceptionReport
from owslib.sos import SensorObservationService as Sos
from owslib.swe.sensor.sml import SensorML
from owslib.util import ServiceException


class IoosSweSos(Collector):
    def __init__(self, url, xml=None):
        super(IoosSweSos, self).__init__()
        self.server = Sos(url, xml=xml)

    def metadata(self, output_format=None, feature_name_callback=None, **kwargs):
        """
        Gets SensorML objects for all procedures in your filtered features.

        You should override the default output_format for servers that do not
        respond properly.
        """
        callback = feature_name_callback or str
        if output_format is None:
            output_format = 'text/xml; subtype="sensorML/1.0.1/profiles/ioos_sos/1.0"'

        responses = []
        if self.features is not None:
            for feature in self.features:
                ds_kwargs = kwargs.copy()
                ds_kwargs.update({'outputFormat': output_format,
                                  'procedure'   : callback(feature)})

                responses.append(SensorML(self.server.describe_sensor(**ds_kwargs)))

        return responses

    def metadata_plus_exceptions(self, output_format=None, feature_name_callback=None, **kwargs):
        """
        Gets SensorML objects for all procedures in your filtered features.

        Return two dictionaries for service responses keyed by 'feature':
            responses: values are SOS DescribeSensor response text
            response_failures: values are exception text content furnished from ServiceException, ExceptionReport

        You should override the default output_format for servers that do not
        respond properly.
        """
        callback = feature_name_callback or str
        if output_format is None:
            output_format = 'text/xml; subtype="sensorML/1.0.1/profiles/ioos_sos/1.0"'

        responses = {}
        response_failures = {}
        if self.features is not None:
            for feature in self.features:
                ds_kwargs = kwargs.copy()
                ds_kwargs.update({'outputFormat': output_format,
                                  'procedure'   : callback(feature)})
                try:
                    responses[feature] = (SensorML(self.server.describe_sensor(**ds_kwargs)))
                except (ServiceException, ExceptionReport) as e:
                    response_failures[feature] = str(e)

        return (responses, response_failures)

    def setup_params(self, **kwargs):
        params = kwargs

        if self.bbox is not None:
            params["featureOfInterest"] = "BBOX:%s,%s,%s,%s" % (self.bbox[0], self.bbox[1], self.bbox[2], self.bbox[3])

        if self.start_time is not None:
            params["eventTime"] = self.start_time.strftime('%Y-%m-%dT%H:%M:%SZ')
        if self.end_time is not None:
            params["eventTime"] += "/%s" % self.end_time.strftime('%Y-%m-%dT%H:%M:%SZ')

        if self.variables is None or len(self.variables) < 1:
            raise ValueError("You must set a filter for at least one variable (observedProperty)")
        else:
            ops = ",".join(self.variables)
            if isinstance(ops, string_types):
                ops = [ops]
            params["observedProperties"] = ops

        return params

    def collect(self, **kwargs):
        # there is an unfortunate difference in how 52N and ncSOS handle the response format.
        # 52N expects subtype, ncSOS expects schema.
        # consult the observed properties and getcaps to figure out which should be used if none passed
        if 'responseFormat' not in kwargs:

            # iterate offerings and see if we need to change to subtype
            off_dict = {off.name : off for off in self.server.offerings}

            response_format = None

            for offering in kwargs.get('offerings', []):
                if offering not in off_dict:
                    continue

                ioos_formats = [rf for rf in off_dict[offering].response_formats if 'ioos_sos/1.0' in rf]
                if not len(ioos_formats):
                    raise Exception("No ioos_sos/1.0 response format found for offering {}".format(offering))

                if response_format != ioos_formats[0]:
                    response_format = ioos_formats[0]

            kwargs["responseFormat"] = response_format

        return IoosGetObservation(self.raw(**kwargs)).observations

    def raw(self, **kwargs):
        params = self.setup_params(**kwargs)
        return self.server.get_observation(**params)



from __future__ import (absolute_import, division, print_function)

import pytz
from owslib.etree import etree

from shapely.geometry import Point as sPoint
from paegan.cdm.dsg.features.base.point import Point
from paegan.cdm.dsg.features.station import Station as Station
from paegan.cdm.dsg.collections.station_collection import StationCollection
from paegan.cdm.dsg.member import Member
from datetime import datetime


class AwcToPaegan(object):
    def __init__(self, awc_list):
        for awc_data in awc_list:
            self._root = etree.fromstring(awc_data.encode())

            '''Code to get station iterator goes here.'''
            stations = []
            station_lookup = []
            times = []

            for metar in self._root.iter('METAR'):
                uid = metar.find("station_id").text
                if uid not in station_lookup:
                    s = Station()
                    s.uid = uid
                    vertical = metar.find("elevation_m").text
                    if vertical is None:
                        vertical = 0
                    s.location = sPoint(float(metar.find("latitude").text), float(metar.find("longitude").text), float(vertical))
                    s.set_property("metar_type", metar.find("metar_type").text)
                    s.set_property("horizontal_crs", "GCS")
                    s.set_property("vertical_units", "m")
                    s.set_property("vertical_crs", "AGL")

                    stations.append(s)
                    station_lookup.append(s.uid)
                    times.append({})
                else:
                    s = stations[station_lookup.index(uid)]

                variables = ["elevation_m", "raw_text", "temp_c", "dewpoint_c", "wind_dir_degrees", "wind_speed_kt", "visibility_statute_mi", "altim_in_hg", "wx_string", "sky_condition", "flight_category"]
                for variable in variables:
                    time_string = metar.find("observation_time").text
                    dt = datetime.strptime(time_string, "%Y-%m-%dT%H:%M:%SZ")
                    if dt.tzinfo is None:
                        dt = dt.replace(tzinfo=pytz.utc)
                    dt = dt.astimezone(pytz.utc)

                    if dt not in list(times[station_lookup.index(s.uid)].keys()):
                        times[station_lookup.index(s.uid)][dt] = []

                    if metar.find(variable) != None:
                        if variable in set(["raw_text", "flight_category", "wx_string"]):
                            times[station_lookup.index(s.uid)][dt].append(Member(value=metar.find(variable).text, unit=None, name=variable, description=variable, standard=None))
                        elif variable == "sky_condition":
                            sky_condition = []
                            for cond in metar.findall(variable):
                                cover = cond.attrib["sky_cover"]
                                try:
                                    alt = cond.attrib["cloud_base_ft_agl"]
                                except:
                                    alt = None
                                sky_condition.append({"sky_cover" : cover, "cloud_base_ft_agl" : alt})
                            times[station_lookup.index(s.uid)][dt].append(Member(value=sky_condition, unit="ft_above_ground_level", name=variable, description=variable, standard=None))
                        else:
                            times[station_lookup.index(s.uid)][dt].append(Member(value=float(metar.find(variable).text), unit=variable.split("_")[-1], name=variable, description=variable, standard=None))

            for time_dict, station in zip(times, stations):
                for dts, members in time_dict.items():
                    p = Point()
                    p.time = dts
                    p.location = station.location
                    p.members = members
                    station.add_element(p)

        self.feature = StationCollection(elements=stations)



from __future__ import (absolute_import, division, print_function)

from pyoos.collectors.collector import Collector
from pyoos.parsers.awc import AwcToPaegan
import requests


class AwcRest(Collector):
    def __init__(self, **kwargs):
        super(AwcRest, self).__init__()
        self.stations_url = 'https://www.aviationweather.gov/docs/metar/stations.txt'
        self.data_url     = 'http://www.aviationweather.gov/adds/dataserver_current/httpparam'

    def get_stations(self):
        if self._features is None:
            r = requests.get(self.stations_url)
            # Arghhhh Fortran-like fixed format!!!
            _stations = [line[20:24] for line in r.text.split('\n')
                         if len(line) == 83 and not line.startswith('!')]
            self._stations = sorted(filter(lambda code: code.strip(), _stations))
            self._features = self._stations
        return self._features

    def set_stations(self, codes):
        self._features = codes
        self._stations = codes
    stations = property(get_stations, set_stations)

    def list_features(self):
        return self.features

    def get_raw_response(self, **kwargs):
        r = []
        if ((self.bbox[3]-self.bbox[1]) * (self.bbox[2]-self.bbox[0])) > 4:
            x = self.bbox[0]
            y = self.bbox[2]
            while x <= self.bbox[2]:
                while y <= self.bbox[3]:
                    kwargs["minLat"] = y
                    kwargs["minLon"] = x
                    kwargs["maxLat"] = y + 2
                    kwargs["maxLon"] = x + 2
                    r.append(requests.get(self.data_url, params=kwargs).text)
                    y += 2
                x += 2
        else:
            r.append(requests.get(self.data_url, params=kwargs).text)
        return r

    def setup_params(self, **kwargs):
        params = kwargs
        params["minLat"] = ''
        params["minLon"] = ''
        params["maxLat"] = ''
        params["maxLon"] = ''
        if self.bbox is not None:  # Must be in format: (minx, miny, maxx, maxy)
            params["minLat"] = str(self.bbox[1])
            params["minLon"] = str(self.bbox[0])
            params["maxLat"] = str(self.bbox[3])
            params["maxLon"] = str(self.bbox[2])
        return params

    def collect(self):
        params = self.setup_params(format="xml", hoursBeforeNow="48", requestType="retrieve", dataSource="metars")
        data = self.get_raw_response(**params)
        return AwcToPaegan(data).feature

    def raw(self, **kwargs):
        params = self.setup_params(format="xml", hoursBeforeNow="48", requestType="retrieve", dataSource="metars")
        data = self.get_raw_response(**params)
        return data



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

from pyoos.utils.etree import etree
from owslib.util import testXMLValue
from paegan.cdm.dsg.features.station import Station
from paegan.cdm.dsg.features.base.point import Point
from paegan.cdm.dsg.collections.station_collection import StationCollection
from shapely.geometry import Point as sPoint

import pytz
from pyoos.utils.asatime import AsaTime
from paegan.cdm.dsg.member import Member


def units(param):
    # http://cdmo.baruch.sc.edu/documents/manual.pdf
    units = {
        # Water Quality
        'Temp'      :       u'°C',
        'SpCond'    :       'mS/cm',
        'Sal'       :       'ppt',
        'DO_pct'    :       '%',
        'DO_mgl'    :       'mg/L',
        'Depth'     :       'm',
        'cDepth'    :       'm',
        'Level'     :       'm',
        'cLevel'    :       'm',
        'pH'        :       '',
        'Turb'      :       'NTU',
        'ChlFluor'  :       u'µg/L',
        # Meteorological
        'ATemp'     :       u'°C',
        'RH'        :       '%',
        'BP'        :       'mb',
        'WSpd'      :       'm/s',
        'MaxWSpd'   :       'm/s',
        'MaxWSpdT'  :       'hh:mm',
        'Wdir'      :       u'°TN',
        'SDWDir'    :       'sd',
        'TotPAR'    :       'mmoles/m^2',
        'TotPrcp'   :       'mm',
        'CumPrcp'   :       'mm',
        'TotSoRad'  :       'watts/m^2',
        # Nutrient and Pigment
        'PO4F'      :       'mg/L',
        'NH4F'      :       'mg/L',
        'NO2F'      :       'mg/L',
        'NO3F'      :       'mg/L',
        'NO23F'     :       'mg/L',
        'CHLA_N'    :       u'µg/L'
    }
    return units.get(param)


def standard(param):
    # http://cdmo.baruch.sc.edu/documents/manual.pdf
    # 10/24/2015, EM: Some of these don't seem to appear on SOAP service; eg, cDepth
    standards = {
        # Water Quality
        'Temp'      :       'sea_water_temperature',
        'SpCond'    :       'specific_conductance',
        'Sal'       :       'sea_water_salinity',
        'DO_pct'    :       'oxygen_concentration_in_sea_water',
        'DO_mgl'    :       'oxygen_concentration_in_sea_water',
        'Depth'     :       'depth',
        'cDepth'    :       'depth',
        'Level'     :       'depth',
        'cLevel'    :       'depth',
        'pH'        :       'sea_water_acidity',
        'Turb'      :       'sea_water_turbidity',
        'ChlFluor'  :       'chlorophyll_fluorescence_in_sea_water',
        # Meteorological
        'ATemp'     :       'air_temperature',
        'RH'        :       'relative_humidity',
        'BP'        :       'air_pressure',
        'WSpd'      :       'wind_speed',
        'MaxWSpd'   :       'wind_speed_of_gust',
        'MaxWSpdT'  :       'time_of_max_wind_speed',
        'Wdir'      :       'wind_direction_from_true_north',
        'SDWDir'    :       'wind_direction_standard_deviation',
        'TotPAR'    :       'total_par_LiCor',
        'TotPrcp'   :       'total_precipitation',
        'CumPrcp'   :       'cumulative_precipitation',
        'TotSoRad'  :       'total_solar_radiation',
        # Nutrient and Pigment
        'PO4F'      :       'orthophosphate_concentration_in_sea_water',
        'NH4F'      :       'ammonium_concentration_in_sea_water',
        'NO2F'      :       'nitrite_concentration_in_sea_water',
        'NO3F'      :       'nitrate_concentration_in_sea_water',
        'NO23F'     :       'nitrite_and_nitrate_concentration_in_sea_water',
        'CHLA_N'    :       'chlorophyll_concentration_in_sea_water'
    }
    return standards.get(param)


class NerrsToPaegan(object):

    def __init__(self, response_list, nerrs_stations=None):
        assert isinstance(response_list, dict)

        if nerrs_stations is None:
            from pyoos.collectors.nerrs.nerrs_soap import NerrsSoap
            nerrs_stations = NerrsSoap().stations

        def get_station(feature):
            for s in nerrs_stations:
                if s['Station_Code'].lower() == feature.lower():
                    return s

        skip_tags = ["DateTimeStamp", "utcStamp", "data", "MaxWSpdT"]

        stations = []
        for feature, response in response_list.items():
            if not isinstance(response, etree._Element):
                response = etree.fromstring(response)

            feature = get_station(feature)

            s = Station()
            s.uid = feature['Station_Code']
            s.name = feature['Station_Name']
            s.location = sPoint(feature['Longitude'], feature['Latitude'], 0)
            s.set_property("state", feature['State'])
            s.set_property("siteid", feature['NERR_Site_ID'])
            s.set_property("horizontal_crs", "EPSG:4326")
            s.set_property("vertical_units", "m")
            s.set_property("vertical_crs", "EPSG:4297")
            s.set_property("location_description", feature['Reserve_Name'])

            for data in response.findall(".//data"):
                p = Point()
                t = AsaTime.parse(testXMLValue(data.find("utcStamp")))
                t = t.replace(tzinfo=pytz.utc)
                p.time = t
                p.location = s.location
                for child in data:
                    if child.tag not in skip_tags:
                        try:
                            val = float(child.text)
                            p.add_member(Member(value=val, name=child.tag, description=child.tag,
                                                unit=units(child.tag), standard=standard(child.tag)))
                        except TypeError:
                            # Value was None
                            pass

                s.add_element(p)

            stations.append(s)

        self.feature = StationCollection(elements=stations)



from __future__ import (absolute_import, division, print_function)
from six import text_type

import itertools

from owslib.swe.sensor.sml import SensorML
from owslib.namespaces import Namespaces
from owslib.util import testXMLValue, testXMLAttribute, nspath_eval

from pyoos.parsers.ioos.describe_sensor import IoosDescribeSensor

from dateutil import parser
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin
from pyoos.utils import ElementType, etree
import warnings


def get_namespaces():
    n = Namespaces()
    return n.get_namespaces(["sml101", "gml", "xlink", "swe101"])

namespaces = get_namespaces()
SWE_NS = namespaces['swe101']
SML_NS = namespaces['sml101']
ont = 'http://mmisw.org/ont/ioos/definition/'


def nsp(path):
    return nspath_eval(path, namespaces)


class DescribeSensor(IoosDescribeSensor):
    @classmethod
    def get_named_by_definition(cls, element_list, string_def):
        """Attempts to get an IOOS definition from a list of xml elements"""
        try:
            return next((st.value for st in element_list
                        if st.definition == string_def))
        except:
            return None

    def get_ioos_def(self, ident, elem_type, ont):
        """Gets a definition given an identifier and where to search for it"""
        if elem_type == 'identifier':
            getter_fn = self.system.get_identifiers_by_name
        elif elem_type == 'classifier':
            getter_fn = self.system.get_classifiers_by_name
        else:
            raise ValueError("Unknown element type '{}'".format(elem_type))
        return DescribeSensor.get_named_by_definition(getter_fn(ident),
                                                      urljoin(ont, ident))

    def __init__(self, element):
        """ Common things between all describe sensor requests """
        if isinstance(element, ElementType):
            root = element
        else:
            root = etree.fromstring(element)

        sml_str = ".//{{{0}}}identifier/{{{0}}}Term[@definition='http://mmisw.org/ont/ioos/definition/%s']".format(SML_NS)

        # TODO: make this cleaner
        if hasattr(root, 'getroot'):
            root = root.getroot()
        self.system = SensorML(element).members[0]

        self.ioos_version = testXMLValue(root.find(".//{%s}field[@name='ioosTemplateVersion']/{%s}Text/{%s}value" % (SWE_NS, SWE_NS, SWE_NS)))
        if self.ioos_version != "1.0":
            warnings.warn("Warning: Unsupported IOOS version (%s). Supported: [1.0]" % self.ioos_version)

        self.shortName = self.get_ioos_def('shortName', 'identifier', ont)
        self.longName = self.get_ioos_def('longName', 'identifier', ont)
        self.keywords = list(map(str, self.system.keywords))

        # Location
        try:
            self.location = self.system.location[0]
        except (TypeError, IndexError):  # No location exists
            self.location = None

        # Timerange
        try:
            timerange      = testXMLValue(self.system.get_capabilities_by_name("observationTimeRange")[0].find(".//" + nsp("swe101:TimeRange/swe101:value"))).split(" ")
            self.starting  = parser.parse(timerange[0])
            self.ending    = parser.parse(timerange[1])
        except (AttributeError, TypeError, ValueError, IndexError):
            self.starting  = None
            self.ending    = None

class GenericSensor(DescribeSensor):
    """A class used primarily for extracting data from non-IOOS SWE datasets"""
    def __init__(self, element):
        super(GenericSensor, self).__init__(element=element)
        # try to find an identifier element of some sort
        self.id = None
        for ident in ['sensorId', 'stationId', 'networkId',
                      'Sensor ID', 'Station ID', 'network ID']:
            res = self.system.get_identifiers_by_name(ident)
            if res:
                self.id = res[0].value
                break
        self.variables = [comp.values()[0] for
                          comp in self.system.components]

class NetworkDS(DescribeSensor):
    def __init__(self, element):
        super(NetworkDS, self).__init__(element=element)

        self.id = self.get_ioos_def("networkID", "identifier", ont)

        # Verbose method of describing members.  Pull out each individual procedure
        # there is no 'lower-case' function in XPath 1, so we have to settle
        # with this
        sml_str = ".//{{{0}}}identifier/{{{0}}}Term[@definition='http://mmisw.org/ont/ioos/definition/%s']".format(SML_NS)
        name_trans = """translate(@name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')"""
        stat_id_val = ".//sml101:identifier[{}='stationid']/sml101:Term/sml101:value".format(name_trans)
        # TODO: maybe refactor to use ioos_get_def instead
        def first_xpath_or_none(e):
            res = e.xpath(stat_id_val, namespaces=namespaces)
            if res:
                return res[0]
            else:
                return None
        self.procedures = sorted(list(set(
                            [testXMLValue(first_xpath_or_none(comp))
                            for comp in self.system.components])))


class StationDS(DescribeSensor):
    def __init__(self, element):
        super(StationDS, self).__init__(element=element)

        self.id = self.get_ioos_def("stationID", "identifier", ont)
        self.platformType = self.get_ioos_def("platformType", "classifier", ont)
        # Verbose method of describing members.  Pull out each individual
        # variable definition
        self.variables = sorted(list(set(
                                 itertools.chain.from_iterable(
                                     [[testXMLAttribute(quan, "definition")
                                      for quan in
                                     comp.findall(".//" +
                                                  nsp("swe101:Quantity"))]
                                      for comp in self.system.components]))))
        # if no variables were picked up, fall back to using original SML
        # components instead
        if not self.variables:
            self.variables = sorted([comp.values()[0] for
                                    comp in self.system.components])


class SensorDS(DescribeSensor):
    def __init__(self, element):
        super(SensorDS, self).__init__(element=element)

        self.id = self.get_ioos_def("sensorID", "identifier", ont)
        self.platformType = self.get_ioos_def("platformType", "classifier", ont)



from __future__ import (absolute_import, division, print_function)

import pytz
import unittest
from datetime import datetime, timedelta
import pytest
from pyoos.collectors.nerrs.nerrs_soap import NerrsSoap


class NerrTest(unittest.TestCase):

    def setUp(self):
        self.c = NerrsSoap()

    @pytest.mark.xfail
    def test_list_nerrs_features(self):
        # The number of features may change... just make sure we get enough
        assert len(self.c.list_features()) > 10

    @pytest.mark.xfail
    def test_list_nerrs_variables(self):
        # The number of variables may change... just make sure we get enough
        assert len(self.c.list_variables()) > 10

        station_vars = self.c.list_variables(feature='rkbfbwq')
        assert station_vars == ['Depth', 'DO_mgl', 'DO_pct', 'pH', 'Sal', 'SpCond', 'Temp', 'Turb']

    @pytest.mark.xfail
    def test_nerrs_variable_filter(self):

        self.c.filter(features=['owcowmet', 'rkbfbwq'])
        self.c.filter(variables=["ATemp"])
        raw = self.c.raw()
        dsg = self.c.collect()
        # 'rkbfbwq' does not have ATemp, so it not returned
        assert len(raw) == 1
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet']

        self.c.clear()
        self.c.filter(features=['owcowmet', 'rkbfbwq'])
        self.c.filter(variables=["ATemp", "Temp"])
        raw = self.c.raw()
        dsg = self.c.collect()
        # both are returned
        assert len(raw) == 2
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet', 'rkbfbwq']

        # Must specify BBOX or Features subset for NERRS
        self.c.clear()
        self.c.filter(variables=["ATemp", "Temp"])
        with pytest.raises(ValueError):
            self.c.raw()

    @pytest.mark.xfail
    def test_nerrs_bbox_filter(self):

        self.c.filter(bbox=(85.0196, 29.6079, 85.089, 29.7791))
        raw = self.c.raw()
        dsg = self.c.collect()
        # Five features in the bounding box
        assert len(raw) == 5
        assert len(dsg.elements) == 5

    @pytest.mark.xfail
    def test_nerrs_feature_filter(self):

        self.c.filter(features=['owcowmet'])
        raw = self.c.raw()
        dsg = self.c.collect()
        assert len(raw) == 1
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet']

        self.c.clear()
        self.c.filter(features=['owcowmet', 'rkbfbwq'])
        raw = self.c.raw()
        dsg = self.c.collect()
        # both are returned
        assert len(raw) == 2
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet', 'rkbfbwq']

    @pytest.mark.xfail
    def test_nerrs_time_filter(self):

        self.c.filter(features=['owcowmet', 'rkbfbwq'])

        ending = datetime.utcnow() - timedelta(days=90)
        ending = ending.replace(tzinfo=pytz.utc)
        starting = ending - timedelta(hours=24)
        starting = starting.replace(tzinfo=pytz.utc)
        self.c.filter(start_time=starting, end_time=ending)
        raw = self.c.raw()
        dsg = self.c.collect()
        # both are returned
        assert len(raw) == 2
        sorted_stations = sorted([x.uid for x in dsg.elements])
        assert sorted_stations == ['owcowmet', 'rkbfbwq']
        station = dsg.elements[0]
        station.calculate_bounds()
        assert sorted(station.time_range)[0] > starting - timedelta(hours=24)
        assert sorted(station.time_range)[0] < ending + timedelta(hours=24)
        assert sorted(station.time_range)[-1] > starting - timedelta(hours=24)
        assert sorted(station.time_range)[-1] < ending + timedelta(hours=24)

    @pytest.mark.xfail
    def test_nerrs_filter_chaining(self):

        # Test NERRS chaining
        self.c.filter(features=['owcowmet', 'rkbfbwq']).filter(variables=["ATemp"])
        raw = self.c.raw()
        dsg = self.c.collect()
        # only 'owcowmet' returning
        assert len(raw) == 1
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet']

        # Test multiple filter types on one call to 'filter'
        self.c.clear()
        self.c.filter(features=['owcowmet', 'rkbfbwq'], variables=["ATemp"])
        raw = self.c.raw()
        dsg = self.c.collect()
        # only 'owcowmet' returning
        assert len(raw) == 1
        assert sorted([x.uid for x in dsg.elements]) == ['owcowmet']



from __future__ import (absolute_import, division, print_function)

__version__ = '0.8.3'

# Package level logger
import logging
try:
    # Python >= 2.7
    from logging import NullHandler
except ImportError:
    # Python < 2.7
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass
logger = logging.getLogger("pyoos")
logger.addHandler(logging.NullHandler())



from __future__ import (absolute_import, division, print_function)

import os.path
import re
import requests
import pytz
from datetime import datetime
from bs4 import BeautifulSoup
from fiona import collection

from pyoos.parsers.hads import HadsParser
from pyoos.collectors.collector import Collector


class Hads(Collector):
    def __init__(self, **kwargs):
        super(Hads, self).__init__()

        self.states_url         = kwargs.get('states_url', "http://amazon.nws.noaa.gov/hads/goog_earth/")
        self.metadata_url       = kwargs.get('metadata_url',  "http://amazon.nws.noaa.gov/nexhads2/servlet/DCPInfo")
        self.obs_retrieval_url  = kwargs.get('obs_retrieval_url', "http://amazon.nws.noaa.gov/nexhads2/servlet/DecodedData")

        self.station_codes      = None
        self.parser             = HadsParser()

    def clear(self):
        super(Hads, self).clear()

        self.station_codes      = None

    @Collector.bbox.setter
    def bbox(self, bbox):
        Collector.bbox.fset(self, bbox)
        self.station_codes      = None

    @Collector.features.setter
    def features(self, features):
        Collector.features.fset(self, features)
        self.station_codes      = None

    def list_variables(self):
        """
        List available variables and applies any filters.
        """
        station_codes = self._get_station_codes()
        station_codes = self._apply_features_filter(station_codes)
        variables = self._list_variables(station_codes)

        if hasattr(self, '_variables') and self.variables is not None:
            variables.intersection_update(set(self.variables))

        return list(variables)

    def _list_variables(self, station_codes):
        """
        Internal helper to list the variables for the given station codes.
        """
        # sample output from obs retrieval:
        #
        # DD9452D0
        #     HP(SRBM5)
        #         2013-07-22 19:30 45.97
        #     HT(SRBM5)
        #         2013-07-22 19:30 44.29
        #     PC(SRBM5)
        #         2013-07-22 19:30 36.19
        #
        rvar = re.compile("""\n\s([A-Z]{2}[A-Z0-9]{0,1})\(\w+\)""")

        variables = set()
        resp = requests.post(self.obs_retrieval_url, data={'state' : 'nil',
                                                           'hsa'   : 'nil',
                                                           'of'    : '3',
                                                           'extraids' : " ".join(station_codes),
                                                           'sinceday' : -1})
        resp.raise_for_status()

        list(map(variables.add, rvar.findall(resp.text)))
        return variables

    def list_features(self):
        station_codes = self._get_station_codes()
        station_codes = self._apply_features_filter(station_codes)

        return station_codes

    def collect(self):
        var_filter = None
        if hasattr(self, '_variables'):
            var_filter = self._variables

        time_extents = (self.start_time if hasattr(self, 'start_time') else None, self.end_time if hasattr(self, 'end_time') else None)

        metadata, raw_data = self.raw()
        return self.parser.parse(metadata, raw_data, var_filter, time_extents)

    def raw(self, format=None):
        """
        Returns a tuple of (metadata, raw data)
        """
        station_codes = self._apply_features_filter(self._get_station_codes())
        metadata      = self._get_metadata(station_codes)
        raw_data      = self._get_raw_data(station_codes)

        return (metadata, raw_data)

    def _apply_features_filter(self, station_codes):
        """
        If the features filter is set, this will return the intersection of
        those filter items and the given station codes.
        """
        # apply features filter
        if hasattr(self, 'features') and self.features is not None:
            station_codes = set(station_codes)
            station_codes = list(station_codes.intersection(set(self.features)))

        return station_codes

    def _get_metadata(self, station_codes):
        resp = requests.post(self.metadata_url, data={'state'    : 'nil',
                                                      'hsa'      : 'nil',
                                                      'of'       : '1',
                                                      'extraids' : " ".join(station_codes),
                                                      'data'     : "Get Meta Data"})
        resp.raise_for_status()
        return resp.text

    def _get_station_codes(self, force=False):
        """
        Gets and caches a list of station codes optionally within a bbox.

        Will return the cached version if it exists unless force is True.
        """
        if not force and self.station_codes is not None:
            return self.station_codes

        state_urls = self._get_state_urls()

        # filter by bounding box against a shapefile
        state_matches = None

        if self.bbox:
            with collection(os.path.join("resources", "ne_50m_admin_1_states_provinces_lakes_shp.shp"), "r") as c:
                geom_matches = [x['properties'] for x in c.filter(bbox=self.bbox)]
                state_matches = [x['postal'] if x['admin'] != 'Canada' else 'CN' for x in geom_matches]

        self.station_codes = []

        for state_url in state_urls:
            if state_matches is not None:
                state_abbr = state_url.split("/")[-1].split(".")[0]
                if state_abbr not in state_matches:
                    continue

            self.station_codes.extend(self._get_stations_for_state(state_url))

        if self.bbox:
            # retreive metadata for all stations to properly filter them
            metadata        = self._get_metadata(self.station_codes)
            parsed_metadata = self.parser._parse_metadata(metadata)

            def in_bbox(code):
                lat = parsed_metadata[code]['latitude']
                lon = parsed_metadata[code]['longitude']

                return lon >= self.bbox[0] and lon <= self.bbox[2] and lat >= self.bbox[1] and lat <= self.bbox[3]

            self.station_codes = list(filter(in_bbox, self.station_codes))

        return self.station_codes

    def _get_state_urls(self):
        root = BeautifulSoup(requests.get(self.states_url).text)
        areas = root.find_all("area")
        return list(set([x.attrs.get('href', None) for x in areas]))

    def _get_stations_for_state(self, state_url):
        state_root = BeautifulSoup(requests.get(state_url).text)
        return [x for x in [x.attrs['href'].split("nesdis_id=")[-1] for x in state_root.find_all('a')] if len(x) > 0]

    def _get_raw_data(self, station_codes):
        since = 7
        if hasattr(self, 'start_time') and self.start_time is not None:
            # calc delta between now and start_time
            timediff = datetime.utcnow().replace(tzinfo=pytz.utc) - self.start_time

            if timediff.days == 0:
                if timediff.seconds / 60 / 60 > 0:
                    since = -(timediff.seconds / 60 / 60)
                elif timediff.seconds / 60 > 0:
                    since = -1  # 1 hour minimum resolution
            else:
                since = min(7, timediff.days)       # max of 7 days

        resp = requests.post(self.obs_retrieval_url, data={'state'    : 'nil',
                                                           'hsa'      : 'nil',
                                                           'of'       : '1',
                                                           'extraids' : " ".join(station_codes),
                                                           'sinceday' : since})
        resp.raise_for_status()

        return resp.text



from __future__ import (absolute_import, division, print_function)

import pytz

class Collector(object):

    def __init__(self):
        self._end_time = None
        self._start_time = None
        self._bbox = None
        self._variables = None
        self._features = None

    def get_start_time(self):
        """
            The start time to collect data from
            This should be overwritten by subclasses
        """
        return self._start_time

    def set_start_time(self, time):
        if time is not None:
            if not time.tzinfo:
                time = time.replace(tzinfo=pytz.utc)
            time = time.astimezone(pytz.utc)
        self._start_time = time
    start_time = property(get_start_time, set_start_time)

    def get_end_time(self):
        """
            The end time to collect data to
        """
        return self._end_time

    def set_end_time(self, time):
        if time is not None:
            if not time.tzinfo:
                time = time.replace(tzinfo=pytz.utc)
            time = time.astimezone(pytz.utc)
        self._end_time = time
    end_time = property(get_end_time, set_end_time)

    def get_bbox(self):
        """
            The bbox to collect data from as a (minx, miny, maxx, maxy) tuple
        """
        return self._bbox

    def set_bbox(self, bbox):
        if bbox is not None:
            if isinstance(bbox, list) or isinstance(bbox, tuple):
                bbox = tuple(bbox)
            else:
                raise ValueError("Not a recognized bbox. \
                                  Must be in format: (minx, miny, maxx, maxy)")
        self._bbox = bbox
    bbox = property(get_bbox, set_bbox)

    def get_variables(self):
        """
            The variables to collect data from as a list of strings.
        """
        return self._variables

    def set_variables(self, variables):
        if variables is not None:
            if isinstance(variables, list) or isinstance(variables, tuple):
                variables = list(variables)
            else:
                raise ValueError("Not a recognized variable list. \
                                  Must be a list or tuple of strings")
        self._variables = variables
    variables = property(get_variables, set_variables)

    def get_features(self):
        """
            The unique features to collect data from as a list of strings.
        """
        return self._features

    def set_features(self, features):
        if features is not None:
            if isinstance(features, list) or isinstance(features, tuple):
                features = list(features)
            else:
                raise ValueError("Not a recognized variable list. \
                                  Must be a list or tuple of strings")
        self._features = features
    features = property(get_features, set_features)

    def filter(self, **kwargs):
        if kwargs.get("bbox"):
            self.bbox = kwargs.pop("bbox")

        if kwargs.get("start"):
            self.start_time = kwargs.pop("start")

        if kwargs.get("end"):
            self.end_time = kwargs.pop("end")

        if kwargs.get("variables"):
            self.variables = kwargs.pop("variables")

        if kwargs.get("features"):
            self.features = kwargs.pop("features")

        if len(kwargs) > 0:
            # Apply custom filters that are left
            for k, v in kwargs.items():
                setattr(self, k, v)

        # Return self to enable chaining
        return self

    def clear(self):
        self._end_time = None
        self._start_time = None
        self._bbox = None
        self._variables = None
        self._features = None

    def list_variables(self):
        """
            Lists the available variables for this collector
            This should be overwritten by subclasses!
        """
        raise NotImplementedError

    def list_features(self):
        """
            Lists the available features for this collector
            This should be overwritten by subclasses!
        """
        raise NotImplementedError

    def collect(self):
        """
            Do the actual work and return a Paegan CDM object
            This should be overwritten by subclasses!
        """
        raise NotImplementedError

    def raw(self, format=None):
        """
            Do the actual work and return the raw response object from
            This should be overwritten by subclasses!
        """
        raise NotImplementedError



from __future__ import (absolute_import, division, print_function)

from pyoos.collectors.collector import Collector
from pyoos.utils.etree import etree
from pyoos.parsers.wqx.wqx_outbound import WqxToPaegan
import requests

class WqpRest(Collector):
    def __init__(self, **kwargs):
        super(WqpRest,self).__init__()
        self.sites_url = kwargs.get('sites_url', "http://www.waterqualitydata.us/Station/search")
        self.results_url = kwargs.get('results_url', "http://www.waterqualitydata.us/Result/search")
        self.characteristics_url = kwargs.get("characteristics_url", "http://www.waterqualitydata.us/Codes/Characteristicname")
        self.characteristic_types_url = kwargs.get("characteristic_types_url", "http://www.waterqualitydata.us/Codes/Characteristictype")

    def get_characterisic_types(self, **kwargs):
        root = etree.fromstring(requests.get(self.characteristic_types_url).text)
        return (x.get('value') for x in root.findall('Code'))

    def get_raw_sites_data(self, **kwargs):
        params = self.setup_params(**kwargs)
        return requests.get(self.sites_url, params=params).text

    def get_raw_results_data(self, **kwargs):
        params = self.setup_params(**kwargs)
        return requests.get(self.results_url, params=params).text

    def set_features(self, features):
        super(WqpRest,self).set_features(features)
        if self._features is not None and len(self._features) > 1:
            raise ValueError("WQP only supports extracting one feature at a time \
                              Please reduce your list to a length of one and try again")
    def get_features(self):
        return self._features
    features = property(get_features, set_features)

    def list_variables(self):
        root = etree.fromstring(requests.get(self.characteristics_url).text)
        return (x.get('value') for x in root.findall('Code'))

    def list_features(self):
        raise NotImplementedError("WQP does not allow returning a list of unique features.")

    def setup_params(self, **kwargs):
        params = kwargs
        if self.start_time is not None:
            params["startDateLo"] = self.start_time.strftime("%m-%d-%Y")
        if self.end_time is not None:
            params["startDateHi"] = self.end_time.strftime("%m-%d-%Y")
        if self.bbox is not None:
            params["bbox"] = ",".join([str(x) for x in self.bbox])
        if self.variables is not None:
            params["characteristicName"] = ";".join(self.variables)
        if self.features is not None:
            params["siteid"] = self.features[0]

        return params

    def collect(self):
        params = self.setup_params(mimeType="xml")
        meta = self.get_raw_sites_data(**params)
        data = self.get_raw_results_data(**params)
        return WqxToPaegan(meta,data).feature

    def raw(self, **kwargs):
        params = self.setup_params(mimeType="xml")
        meta = self.get_raw_sites_data(**params)
        data = self.get_raw_results_data(**params)
        return meta, data




from __future__ import (absolute_import, division, print_function)
from six import text_type

from owslib.util import nspath as nsp
from owslib.util import testXMLValue
from pyoos.utils.asatime import AsaTime
from pyoos.utils.etree import etree
import pytz

from shapely.geometry import Point as sPoint
from paegan.cdm.dsg.features.base.point import Point
from paegan.cdm.dsg.features.station import Station as Station
from paegan.cdm.dsg.collections.station_collection import StationCollection
from paegan.cdm.dsg.member import Member


class WqxToPaegan(object):
    """
        Convert a WqxOutbound object or WQX string/element to the Paegan DSG as a StationCollection
    """
    def __init__(self, wqx_metadata, wqx_data):
        if not isinstance(wqx_metadata, WqxOutbound):
            wqx_metadata = WqxOutbound(wqx_metadata)

        if not isinstance(wqx_data, WqxOutbound):
            wqx_data = WqxOutbound(wqx_data)

        if wqx_data.failed or wqx_metadata.failed:
            self.feature = None
        else:
            stations = []
            station_lookup = []
            # Create a station for every MonitoringLocation
            for org in wqx_metadata.organizations:
                for ml in org.locations:
                    s = Station()
                    s.uid = ml.id
                    s.name = ml.name
                    s.set_property("station_type", ml.type)
                    s.set_property("location_description", ml.description)
                    s.set_property("huc", ml.huc)
                    s.set_property("county", ml.county)
                    s.set_property("state", ml.state)
                    s.set_property("country", ml.country)
                    s.set_property("organization_id", org.description.id)
                    s.set_property("organization_name", org.description.name)
                    s.set_property("vertical_units", ml.vertical_measure_units)
                    s.set_property("horizontal_crs", ml.horizontal_crs_name)
                    s.set_property("vertical_crs", ml.vertical_crs_name)

                    # Now set the station's location
                    vertical = 0
                    try:
                        vertical = float(ml.vertical_measure_value)
                    except:
                        pass

                    # convert the vertical to meters if it is ft (which it always is)
                    if ml.vertical_measure_units == "ft":
                        vertical /= 3.28084
                        s.set_property("vertical_units", "m")
                    s.location = sPoint(float(ml.longitude), float(ml.latitude), vertical)

                    stations.append(s)
                    station_lookup.append(s.uid)

            for org in wqx_data.organizations:
                for a in org.activities:
                    p = Point()
                    p.time = a.start_time
                    for r in a.results:
                        p.add_member(Member(value=r.value, unit=r.units, name=r.name, description=r.short_name, standard=None, quality=r.quality, method_id=a.method_id, method_name=a.method_name))

                    # Assign data to the correct station pulled from the metadata
                    station = stations[station_lookup.index(a.location_id)]
                    p.location = station.location
                    station.add_element(p)

            self.feature = StationCollection(elements=stations)


class WqxOutbound(object):
    """
        An WQX formatted <wqx:WQX> block
    """
    def __init__(self, element):
        wqx_ns = "http://qwwebservices.usgs.gov/schemas/WQX-Outbound/2_0/"

        if isinstance(element, text_type):
            try:
                self._root = etree.fromstring(element)
            except ValueError:
                # Strip out the XML header due to UTF8 encoding declaration
                self._root = etree.fromstring(element[38:])
        else:
            self._root = element

        if hasattr(self._root, 'getroot'):
            self._root = self._root.getroot()

        self.failed = False
        self.organizations = []
        orgs = self._root.findall(nsp("Organization", wqx_ns))

        if orgs is None:
            self.failed = True
        else:
            for org in orgs:
                self.organizations.append(WqxOrganization(org, wqx_ns))


class WqxOrganization(object):
    """
        An WQX formatted <wqx:Organization> block
    """
    def __init__(self, element, wqx_ns):
        self._root = element

        self.description = WqxOrganizationDescription(self._root.find(nsp("OrganizationDescription", wqx_ns)), wqx_ns)

        self.locations = []
        mls = self._root.findall(nsp("MonitoringLocation", wqx_ns))
        for loc in mls:
            if loc is not None:
                self.locations.append(WqxMonitoringLocation(loc, wqx_ns))

        self.activities = []
        for act in self._root.findall(nsp("Activity", wqx_ns)):
            self.activities.append(WqxActivity(act, wqx_ns))


class WqxActivity(object):
    """
        An WQX formatted <wqx:Activity> block
    """
    def __init__(self, element, wqx_ns):
        self._root = element

        des = self._root.find(nsp("ActivityDescription", wqx_ns))
        if des is not None:
            self.id = testXMLValue(des.find(nsp("ActivityIdentifier", wqx_ns)))
            self.type = testXMLValue(des.find(nsp("ActivityTypeCode", wqx_ns)))
            self.media = testXMLValue(des.find(nsp("ActivityMediaName", wqx_ns)))

        # Date/Time
        sd = testXMLValue(des.find(nsp("ActivityStartDate", wqx_ns)))  # YYYY-MM-DD
        parse_string = "%s" % sd

        st = des.find(nsp("ActivityStartTime", wqx_ns))
        # If no time is defined, skip trying to pull it out and just use the date
        if st is not None:
            t = testXMLValue(st.find(nsp("Time", wqx_ns)))
            tz = testXMLValue(st.find(nsp("TimeZoneCode", wqx_ns)))

            parse_string = "%s %s" % (parse_string, t)
            if tz is not None:
                parse_string = "%s %s" % (parse_string, tz)

        self.start_time = AsaTime.parse(parse_string)
        if self.start_time.tzinfo is None:
            self.start_time = self.start_time.replace(tzinfo=pytz.utc)

        self.project = testXMLValue(des.find(nsp("ProjectIdentifier", wqx_ns)))
        self.location_id = testXMLValue(des.find(nsp("MonitoringLocationIdentifier", wqx_ns)))
        self.comment = testXMLValue(des.find(nsp("ActivityCommentText", wqx_ns)))

        self.method_id = None
        self.method_name = None
        self.method_context = None
        # Method
        smpl = self._root.find(nsp("SampleDescription", wqx_ns))
        if smpl is not None:
            self.sample_collection_equipment_name = testXMLValue(smpl.find(nsp("SampleCollectionEquipmentName", wqx_ns)))
            smplcol = smpl.find(nsp("SampleCollectionMethod", wqx_ns))
            if smplcol is not None:
                self.method_id = testXMLValue(smplcol.find(nsp("MethodIdentifier", wqx_ns)))
                self.method_context = testXMLValue(smplcol.find(nsp("MethodIdentifierContext", wqx_ns)))
                self.method_name = testXMLValue(smplcol.find(nsp("MethodName", wqx_ns)))

        self.results = []
        for res in self._root.findall(nsp("Result", wqx_ns)):
            self.results.append(WqxResult(res, wqx_ns))


class WqxResult(object):
    def __init__(self, element, wqx_ns):
        self._root = element

        des = self._root.find(nsp("ResultDescription", wqx_ns))
        self.name = None
        self.short_name = None
        self.status = None
        self.stastistical_base_code = None
        self.value_type = None
        self.weight_basis = None
        self.time_basis = None
        self.temperature_basis = None
        if des is not None:
            self.name = testXMLValue(des.find(nsp("CharacteristicName", wqx_ns)))
            self.short_name = testXMLValue(des.find(nsp("ResultSampleFractionText", wqx_ns)))
            self.status = testXMLValue(des.find(nsp("ResultStatusIdentifier", wqx_ns)))
            self.stastistical_base_code = testXMLValue(des.find(nsp("StatisticalBaseCode", wqx_ns)))
            self.value_type = testXMLValue(des.find(nsp("ResultValueTypeName", wqx_ns)))
            self.weight_basis = testXMLValue(des.find(nsp("ResultWeightBasisText", wqx_ns)))
            self.time_basis = testXMLValue(des.find(nsp("ResultTimeBasisText", wqx_ns)))
            self.temperature_basis = testXMLValue(des.find(nsp("ResultTemperatureBasisText", wqx_ns)))

        rm = des.find(nsp("ResultMeasure", wqx_ns))
        self.value = None
        self.units = None
        if rm is not None:
            self.value = testXMLValue(rm.find(nsp("ResultMeasureValue", wqx_ns)))
            self.units = testXMLValue(rm.find(nsp("MeasureUnitCode", wqx_ns)))

        qu = des.find(nsp("DataQuality", wqx_ns))
        self.quality = None
        if qu is not None:
            self.quality = testXMLValue(qu.find(nsp("PrecisionValue", wqx_ns)))

        am = self._root.find(nsp("ResultAnalyticalMethod", wqx_ns))
        self.analytical_method_id = None
        self.analytical_method_id_context = None
        if am is not None:
            self.analytical_method_id = testXMLValue(am.find(nsp("MethodIdentifier", wqx_ns)))
            self.analytical_method_id_context = testXMLValue(am.find(nsp("MethodIdentifierContext", wqx_ns)))

        # Skipping <ResultLabInformation> for now.


class WqxOrganizationDescription(object):
    """
        An WQX formatted <wqx:OrganizationDescription> block
    """
    def __init__(self, element, wqx_ns):
        self._root = element

        self.id = testXMLValue(self._root.find(nsp("OrganizationIdentifier", wqx_ns)))
        self.name = testXMLValue(self._root.find(nsp("OrganizationFormalName", wqx_ns)))


class WqxMonitoringLocation(object):
    """
        An WQX formatted <wqx:MonitoringLocation> block
    """
    def __init__(self, element, wqx_ns):
        self._root = element

        self.id = None
        self.name = None
        self.type = None
        self.description = None
        self.huc = None
        identity = self._root.find(nsp("MonitoringLocationIdentity", wqx_ns))
        if identity is not None:
            self.id = testXMLValue(identity.find(nsp("MonitoringLocationIdentifier", wqx_ns)))
            self.name = testXMLValue(identity.find(nsp("MonitoringLocationName", wqx_ns)))
            self.type = testXMLValue(identity.find(nsp("MonitoringLocationTypeName", wqx_ns)))
            self.description = testXMLValue(identity.find(nsp("MonitoringLocationDescriptionText", wqx_ns)))
            self.huc = testXMLValue(identity.find(nsp("HUCEightDigitCode", wqx_ns)))

        self.latitude = None
        self.longitude = None
        self.map_scale = None
        self.horizontal_collection_method = None
        self.horizontal_crs_name = None
        self.horizontal_crs = None
        self.vertical_crs_name = None
        self.vertical_crs = None
        geo = self._root.find(nsp("MonitoringLocationGeospatial", wqx_ns))
        if geo is not None:
            self.latitude = testXMLValue(geo.find(nsp("LatitudeMeasure", wqx_ns)))
            self.longitude = testXMLValue(geo.find(nsp("LongitudeMeasure", wqx_ns)))
            self.map_scale = testXMLValue(geo.find(nsp("SourceMapScaleNumeric", wqx_ns)))
            self.horizontal_collection_method = testXMLValue(geo.find(nsp("HorizontalCollectionMethodName", wqx_ns)))
            self.horizontal_crs_name = testXMLValue(geo.find(nsp("HorizontalCoordinateReferenceSystemDatumName", wqx_ns)))
            # self.horizontal_crs = Crs("EPSG:" + testXMLValue(geo.find(nsp("HorizontalCoordinateReferenceSystemDatumName", wqx_ns))))
            self.vertical_crs_name = testXMLValue(geo.find(nsp("VerticalCollectionMethodName", wqx_ns)))
            # self.vertical_crs = Crs(testXMLValue("EPSG:" + geo.find(nsp("VerticalCoordinateReferenceSystemDatumName", wqx_ns))))

        self.vertical_measure_value = None
        self.vertical_measure_units = None
        vm = geo.find(nsp("VerticalMeasure", wqx_ns))
        if vm is not None:
            self.vertical_measure_value = testXMLValue(vm.find(nsp("MeasureValue", wqx_ns)))
            self.vertical_measure_units = testXMLValue(vm.find(nsp("MeasureUnitCode", wqx_ns)))

        self.country = testXMLValue(geo.find(nsp("CountryCode", wqx_ns)))
        self.state = testXMLValue(geo.find(nsp("StateCode", wqx_ns)))
        self.county = testXMLValue(geo.find(nsp("CountyCode", wqx_ns)))



from __future__ import (absolute_import, division, print_function)

from pyoos.collectors.ioos.swe_sos import IoosSweSos


class CoopsSos(IoosSweSos):
    def __init__(self, **kwargs):
        kwargs["url"] = 'https://opendap.co-ops.nos.noaa.gov/ioos-dif-sos/SOS'
        super(CoopsSos, self).__init__(**kwargs)
        self._datum = None
        self._dataType = None

    def set_datatype(self, datatype):
        self._dataType = datatype

    def get_datatype(self):
        return self._dataType
    dataType = property(get_datatype, set_datatype)

    def set_datum(self, datum):
        self._datum = datum

    def get_datum(self):
        return self._datum
    datum = property(get_datum, set_datum)

    def list_datatypes(self):
        # TODO: Get from GetCaps
        return ["PreliminarySixMinute",
                "PreliminaryOneMinute",
                "VerifiedSixMinute",
                "VerifiedHourlyHeight",
                "VerifiedHighLow",
                "VerifiedDailyMean",
                "SixMinuteTidePredictions",
                "HourlyTidePredictions",
                "HighLowTidePredictions"]

    def list_datums(self):
        # TODO: Get from GetCaps
        return ["MLLW",
                "MSL",
                "MHW",
                "STND",
                "IGLD",
                "NAVD"]

    def setup_params(self, **kwargs):
        params = super(CoopsSos, self).setup_params(**kwargs)

        if self.features is None or len(self.features) != 1:
            params["offerings"] = ["urn:ioos:network:NOAA.NOS.CO-OPS:All"]
        elif len(self.features) == 1:
            params["offerings"] = ["urn:ioos:station:NOAA.NOS.CO-OPS:%s" % self.features[0]]

        if self.datum is not None:
            if self.datum in self.list_datums():
                if self.datum == "NAVD":
                    params["result"] = "VerticalDatum==urn:ogc:def:datum:epsg::5103"
                else:
                    params["result"] = "VerticalDatum==urn:ioos:def:datum:noaa::%s" % self.datum
            else:
                raise ValueError("Not a supported datum.  Use list_datums() for options")

        if self.dataType is not None:
            if self.dataType in self.list_datatypes():
                params["dataType"] = self.dataType
            else:
                raise ValueError("Not a supported data type.  Use list_datatypes() for options")

        if params.get("responseFormat", None) is None:
            params["responseFormat"] = 'text/csv'

        return params

    def metadata(self, **kwargs):
        callback = lambda x: "urn:ioos:station:NOAA.NOS.CO-OPS:%s" % x
        return super(CoopsSos, self).metadata(feature_name_callback=callback, **kwargs)



from __future__ import (absolute_import, division, print_function)

from pyoos.utils.etree import ElementType, etree
from owslib.namespaces import Namespaces
from owslib.util import testXMLValue

ns = Namespaces()


class IoosGetObservation(object):
    def __new__(cls, element):
        if isinstance(element, ElementType):
            root = element
        else:
            root = etree.fromstring(element)

        if hasattr(root, 'getroot'):
            root = root.getroot()

        XLINK_NS = ns.get_namespace("xlink")
        GML_NS = [ns.get_versioned_namespace('gml', '3.1.1')]
        version = None
        for g in GML_NS:
            try:
                version = testXMLValue(root.find("{%s}metaDataProperty[@{%s}title='ioosTemplateVersion']/{%s}version" % (g, XLINK_NS, g)))
                break
            except:
                continue

        if version == "1.0":
            from pyoos.parsers.ioos.one.get_observation import GetObservation as GO10
            return super(IoosGetObservation, cls).__new__(GO10)
        else:
            raise ValueError("Unsupported IOOS version {}.  Supported: [1.0]".format(version))

    def __init__(self, element):
        # Get individual om:Observations has a hash or name:ob.
        if isinstance(element, ElementType):
            self._root = element
        else:
            self._root = etree.fromstring(element)

        if hasattr(self._root, 'getroot'):
            self._root = self._root.getroot()

        self.observations = []



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

import unittest
from pytest import raises
from pyoos.collectors.wqp.wqp_rest import WqpRest
from pyoos.parsers.wqx.wqx_outbound import WqxOutbound


class WqpTest(unittest.TestCase):

    def setUp(self):
        self.c = WqpRest()

    def test_wqp_sites_metadata(self):
        """
        <WQX xmlns="http://qwwebservices.usgs.gov/schemas/WQX-Outbound/2_0/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://qwwebservices.usgs.gov/schemas/WQX-Outbound/2_0/ http://qwwebservices.usgs.gov/schemas/WQX-Outbound/2_0/index.xsd">
            <Organization>
                <OrganizationDescription>
                    <OrganizationIdentifier>21IOWA</OrganizationIdentifier>
                    <OrganizationFormalName>Iowa Dept. of  Natural Resources</OrganizationFormalName>
                </OrganizationDescription>
                <MonitoringLocation>
                    <MonitoringLocationIdentity>
                        <MonitoringLocationIdentifier>21IOWA-10070005</MonitoringLocationIdentifier>
                        <MonitoringLocationName>Cedar River Upstream of Waterloo/Cedar Falls</MonitoringLocationName>
                        <MonitoringLocationTypeName>River/Stream</MonitoringLocationTypeName>
                        <MonitoringLocationDescriptionText>Below the dam of Old Highway 218 in Cedar Falls.¿Upstream City Site.</MonitoringLocationDescriptionText>
                        <HUCEightDigitCode>07080205</HUCEightDigitCode>
                    </MonitoringLocationIdentity>
                    <MonitoringLocationGeospatial>
                        <LatitudeMeasure>42.5392</LatitudeMeasure>
                        <LongitudeMeasure>-92.4495</LongitudeMeasure>
                        <SourceMapScaleNumeric>000</SourceMapScaleNumeric>
                        <HorizontalCollectionMethodName>Interpolation-Map</HorizontalCollectionMethodName>
                        <HorizontalCoordinateReferenceSystemDatumName>NAD27</HorizontalCoordinateReferenceSystemDatumName>
                        <VerticalMeasure>
                            <MeasureValue>854</MeasureValue>
                            <MeasureUnitCode>ft</MeasureUnitCode>
                        </VerticalMeasure>
                        <VerticalCollectionMethodName>Topographic Map Interpolation</VerticalCollectionMethodName>
                        <VerticalCoordinateReferenceSystemDatumName>NGVD29</VerticalCoordinateReferenceSystemDatumName>
                        <CountryCode>US</CountryCode>
                        <StateCode>19</StateCode>
                        <CountyCode>013</CountyCode>
                    </MonitoringLocationGeospatial>
                </MonitoringLocation>
            </Organization>
        </WQX>
        """
        self.c.filter(features=["21IOWA-10070005"])
        meta, data = self.c.raw()

        org = WqxOutbound(meta).organizations[0]

        # OrganizationDescription
        assert org.description.name == u"Iowa Dept. of  Natural Resources"
        assert org.description.id == u"21IOWA"

        # MonitoringLocation
        assert org.locations[0].id == u"21IOWA-10070005"
        assert org.locations[0].name == u"Cedar River Upstream of Waterloo/Cedar Falls"
        assert org.locations[0].type == u"River/Stream"
        assert org.locations[0].description.startswith(u"Below the dam of Old Highway 218 in Cedar Falls")
        assert org.locations[0].huc == u"07080205"
        assert org.locations[0].latitude == u"42.5392"
        assert org.locations[0].longitude == u"-92.4495"
        assert org.locations[0].vertical_measure_value == u"854"
        assert org.locations[0].vertical_measure_units == u"ft"
        assert org.locations[0].country == u"US"
        assert org.locations[0].state == u"19"
        assert org.locations[0].county == u"013"

    def test_wqp_results_metadata(self):
        self.c.filter(features=["21IOWA-10070005"])
        meta, data = self.c.raw()

        # OrganizationDescription
        org = WqxOutbound(data).organizations[0]
        assert org.description.name == u"Iowa Dept. of  Natural Resources"
        assert org.description.id == u"21IOWA"

    def test_bad_wqp_site(self):
        self.c.filter(features=["s"])
        rmeta, data = self.c.raw()

        with raises(AttributeError):
            print(data.organization)

    def test_into_dsg(self):
        # First feature filter
        self.c.filter(features=["21IOWA-10070005"])
        station_collection = self.c.collect()
        station_collection.calculate_bounds()

        assert station_collection.size == 1
        station = station_collection.elements[0]
        assert len(station.get_unique_members()) == 240
        assert station.location.x == -92.4495
        assert station.location.y == 42.5392
        assert station.location.z == float(854 / 3.28084)

        # Second variable filter
        self.c.filter(variables=["Mercury"])
        station_collection = self.c.collect()
        station_collection.calculate_bounds()

        assert station_collection.size == 1
        station = station_collection.elements[0]
        assert len(station.get_unique_members()) == 1



from __future__ import (absolute_import, division, print_function)

from pyoos.parsers.nerrs import NerrsToPaegan
from pyoos.collectors.collector import Collector
from pyoos.utils.etree import etree
from owslib.util import testXMLValue

from shapely.geometry import Point, box
import requests


class NerrsSoap(Collector):
    def __init__(self, **kwargs):
        """
        :param wildcard: string for optional token-based access mechanism.
        """
        super(NerrsSoap, self).__init__()
        self.wsdl_url = 'http://cdmo.baruch.sc.edu/webservices2/requests.cfc?wsdl'
        self.wildcard = kwargs.get("wildcard")
        self.stations = self.get_stations()

    def get_station(self, feature):
        for s in self.stations:
            if s['Station_Code'].lower() == feature.lower():
                return s

    def get_stations(self):
        if self.wildcard is not None:
            xml_str = """
            <exportStationCodesXMLNew xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                <wildcard xsi:type="xsd:string">OPTIONALVALUE</wildcard>
            </exportStationCodesXMLNew>
            """
            xml_obj = etree.fromstring(xml_str)
            xml_obj.find(".//wildcard").text = self.wildcard
        else:
            """<exportStationCodesXMLNew />"""
            xml_obj = etree.Element("exportStationCodesXMLNew")

        env = self._makesoap(xml_obj)

        stats = []
        for data in env.findall(".//data"):
            s = {}
            for child in data:
                val = testXMLValue(child)
                if val is None:
                    val = ""
                s[child.tag] = val
            lon = float(s['Longitude'])
            s['Longitude'] = lon if lon < 0 else -lon
            s['Latitude'] = float(s['Latitude'])
            stats.append(s)

        return stats

    def _makesoap(self, xmlelement):
        request = """<?xml version="1.0" encoding="UTF-8"?>
        <SOAP-ENV:Envelope  SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
                            xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
                            xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                            xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/">
            <SOAP-ENV:Body>
                REQUEST
            </SOAP-ENV:Body>
        </SOAP-ENV:Envelope>
        """
        enve = etree.fromstring(request)

        body = enve.find(".//{%s}Body" % "http://schemas.xmlsoap.org/soap/envelope/")
        body.append(xmlelement)

        headers = {
            "SOAPAction"        : "\"\"",
        }
        r = requests.post(self.wsdl_url, data=etree.tostring(enve), headers=headers)
        return etree.fromstring(r.text[38:]).find(".//returnData")

    def list_features(self):
        return sorted([s['Station_Code'] for s in self.stations], key=str.lower)

    def list_variables(self, feature=None):
        ignore_vars = ["", None]
        if feature is None:
            stationvars = [s['Params_Reported'].split(",") for s in self.stations]
            # Combine the sublists, ignoring bad names.
            allvars = [v for sublist in stationvars for v in sublist if v not in ignore_vars]
            # Unique the var list.
            return sorted(list(set(allvars)), key=str.lower)
        else:
            s = self.get_station(feature)
            return sorted([v for v in list(set(s['Params_Reported'].split(","))) if v not in ignore_vars], key=str.lower)

    def collect(self):
        results = self.raw()
        return NerrsToPaegan(results, nerrs_stations=self.stations).feature

    def raw(self, **kwargs):
        # These are the features we will actually query against
        query_features = []

        if self.bbox is None and self.features is None:
            raise ValueError("NERRS requires a BBOX or Feature filter")

        if self.bbox is not None and self.features is not None:
            print("Both a BBox and Feature filter is defined, BBOX takes precedence.")

        # BBox takes precedence over features
        if self.bbox is not None:
            test_box = box(self.bbox[0], self.bbox[1], self.bbox[2], self.bbox[3])
            # Set the features and call collect again
            for s in self.stations:
                p = Point(float(s["Longitude"]), float(s["Latitude"]))
                if test_box.intersects(p):
                    query_features.append(s['Station_Code'])
        else:
            query_features = self.features

        if query_features is not None and len(query_features) > 0:

            results = {}
            for f in query_features:
                if self.start_time is not None and self.end_time is not None:
                    # Date range query
                    soap_env = self._build_exportAllParamsDateRangeXMLNew(f)
                else:
                    # Not a date range query
                    soap_env = self._build_exportSingleParamXMLNew(f)

                if soap_env is not None:
                    response = self._makesoap(soap_env)
                    results[f] = etree.tostring(response)

            return results

        return None

    def _build_exportAllParamsDateRangeXMLNew(self, feature):
        xml_str = """
        <exportAllParamsDateRangeXMLNew xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
            <station_code xsi:type="xsd:string">owcowmet</station_code>
            <mindate xsi:type="xsd:string">05/29/2013</mindate>
            <maxdate xsi:type="xsd:string">05/31/2013</maxdate>
            <param xsi:type="xsd:string">WSpd,WDir</param>
            <wildcard xsi:type="xsd:string">OPTIONALVALUE</wildcard>
        </exportAllParamsDateRangeXMLNew>
        """
        xml_obj = etree.fromstring(xml_str)

        if self.wildcard is not None:
            xml_obj.find(".//wildcard").text = self.wildcard

        xml_obj.find(".//mindate").text = self.start_time.strftime('%m/%d/%Y')
        xml_obj.find(".//maxdate").text = self.end_time.strftime('%m/%d/%Y')

        feature_vars = self.list_variables(feature=feature)
        if len(feature_vars) == 0:
            # This feature has no variables, skip it
            return None
        else:
            if self.variables is not None:
                # Query for vars this station has
                queryvars = set(self.variables).intersection(feature_vars)
                if len(queryvars) == 0:
                    # Skip this feature.. it doesn't have any of the requested variables
                    return None
                xml_obj.find(".//param").text = ",".join(queryvars)
            else:
                # No variable subset requested
                xml_obj.find(".//param").text = ",".join(feature_vars)

        # Set station and recs
        xml_obj.find(".//station_code").text = feature
        return xml_obj

    def _build_exportSingleParamXMLNew(self, feature):
        xml_str = """
        <exportSingleParamXMLNew xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
            <station_code xsi:type="xsd:string">FILLME</station_code>
            <recs xsi:type="xsd:string">FILLME</recs>
            <param xsi:type="xsd:string">FILLME</param>
            <wildcard xsi:type="xsd:string">OPTIONALVALUE</wildcard>
        </exportSingleParamXMLNew>
        """
        xml_obj = etree.fromstring(xml_str)

        if self.wildcard is not None:
            xml_obj.find(".//wildcard").text = self.wildcard

        # Set parameters
        feature_vars = self.list_variables(feature=feature)
        if len(feature_vars) == 0:
            # This feature has no variables, skip it
            return None
        else:
            if self.variables is not None:
                # Query for vars this station has
                queryvars = set(self.variables).intersection(feature_vars)
                if len(queryvars) == 0:
                    # Skip this feature.. it doesn't have any of the requested variables
                    return None
                xml_obj.find(".//param").text = ",".join(queryvars)
            else:
                # No variable subset requested
                xml_obj.find(".//param").text = ",".join(feature_vars)

        # Set station and recs
        xml_obj.find(".//station_code").text = feature
        xml_obj.find(".//recs").text = "100"
        return xml_obj
